@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION @ISA $DEBUG);
-$VERSION        = '4.001_050';
+$VERSION        = '4.001_051';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -68,25 +68,21 @@ require Marpa::R3::Common;
 ( $version_result = version_ok($Marpa::R3::Common::VERSION) )
     and die 'Marpa::R3::Common::VERSION ', $version_result;
 
-require Marpa::R3::Value;
-( $version_result = version_ok($Marpa::R3::Value::VERSION) )
-    and die 'Marpa::R3::Value::VERSION ', $version_result;
-
 require Marpa::R3::MetaG;
 ( $version_result = version_ok($Marpa::R3::MetaG::VERSION) )
     and die 'Marpa::R3::MetaG::VERSION ', $version_result;
 
 require Marpa::R3::SLG;
-( $version_result = version_ok($Marpa::R3::Scanless::G::VERSION) )
-    and die 'Marpa::R3::Scanless::G::VERSION ', $version_result;
+( $version_result = version_ok($Marpa::R3::Grammar::VERSION) )
+    and die 'Marpa::R3::Grammar::VERSION ', $version_result;
 
 require Marpa::R3::SLR;
-( $version_result = version_ok($Marpa::R3::Scanless::R::VERSION) )
-    and die 'Marpa::R3::Scanless::R::VERSION ', $version_result;
+( $version_result = version_ok($Marpa::R3::Recognizer::VERSION) )
+    and die 'Marpa::R3::Recognizer::VERSION ', $version_result;
 
 require Marpa::R3::SLV;
-( $version_result = version_ok($Marpa::R3::Scanless::V::VERSION) )
-    and die 'Marpa::R3::Scanless::V::VERSION ', $version_result;
+( $version_result = version_ok($Marpa::R3::Valuer::VERSION) )
+    and die 'Marpa::R3::Valuer::VERSION ', $version_result;
 
 require Marpa::R3::MetaAST;
 ( $version_result = version_ok($Marpa::R3::MetaAST::VERSION) )
@@ -96,6 +92,10 @@ require Marpa::R3::ASF;
 ( $version_result = version_ok($Marpa::R3::ASF::VERSION) )
     and die 'Marpa::R3::ASF::VERSION ', $version_result;
 
+require Marpa::R3::ASF2;
+( $version_result = version_ok($Marpa::R3::ASF2::VERSION) )
+    and die 'Marpa::R3::ASF2::VERSION ', $version_result;
+
 require Marpa::R3::X;
 ( $version_result = version_ok($Marpa::R3::X::VERSION) )
     and die 'Marpa::R3::X::VERSION ', $version_result;
