@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_050';
+$VERSION        = '4.001_051';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -28,12 +28,12 @@ use English qw( -no_match_vars );
 
 sub new {
     my ( $class, $p_rules_source ) = @_;
-    my $meta_recce = Marpa::R3::Internal::Scanless::meta_recce();
+    my $meta_recce = Marpa::R3::Internal::meta_recce();
     my $valuer;
     eval { $meta_recce->read($p_rules_source) }
       or Marpa::R3::exception( "Parse of BNF/Scanless source failed\n",
         $EVAL_ERROR );
-    $valuer = Marpa::R3::Scanless::V->new( { recognizer => $meta_recce } );
+    $valuer = Marpa::R3::Valuer->new( { recognizer => $meta_recce } );
     my $ambiguity_level = $valuer->ambiguity_level();
     if ( $ambiguity_level != 1 ) {
         my $ambiguity_status = $valuer->ambiguous();
@@ -1757,21 +1757,6 @@ sub combine {
     return bless $self, $class;
 } ## end sub combine
 
-sub Marpa::R3::Internal::MetaAST::char_class_to_re {
-    my ($cc_components) = @_;
-    die if ref $cc_components ne 'ARRAY';
-    my ( $char_class, $flags ) = @{$cc_components};
-    $flags = $flags ? '(' . q{?} . $flags . ')' : q{};
-    my $regex;
-    my $error;
-    if ( not defined eval { $regex = qr/$flags$char_class/xms; 1; } ) {
-        $error = qq{Problem in evaluating character class: "$char_class"\n};
-        $error .= qq{  Flags were "$flags"\n} if $flags;
-        $error .= $EVAL_ERROR;
-    }
-    return $regex, $error;
-}
-
 sub Marpa::R3::Internal::MetaAST::flag_string_to_flags {
     my ($raw_flag_string) = @_;
     return q{} if not $raw_flag_string;
@@ -1817,14 +1802,6 @@ sub char_class_to_symbol {
 
         my $cc_components = [ $unmodified_char_class, $flags ];
 
- # Fast fail on badly formed char_class -- we re-evaluate the regex just in time
- # before we register characters.
-        my ( $regex, $eval_error ) =
-          Marpa::R3::Internal::MetaAST::char_class_to_re($cc_components);
-        Carp::croak( 'Bad Character class: ',
-            $char_class, "\n", 'Perl said ', $eval_error )
-          if not $regex;
-
         $symbol = Marpa::R3::Internal::MetaAST::Symbol_List->new($symbol_name);
         $cc_hash->{$symbol_name} = [ $cc_components, $symbol ];
 
