@@ -9,34 +9,34 @@
 # or implied warranties. For details, see the full text of
 # of the licenses in the directory LICENSES.
 
-package Marpa::R3::Scanless::V;
+package Marpa::R3::Valuer;
 
 use 5.010001;
 use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_050';
+$VERSION        = '4.001_051';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
 ## use critic
 
-package Marpa::R3::Internal::Scanless::V;
+package Marpa::R3::Internal_V;
 
 use Scalar::Util qw(blessed tainted);
 use English qw( -no_match_vars );
 
-our $PACKAGE = 'Marpa::R3::Scanless::V';
+our $PACKAGE = 'Marpa::R3::Valuer';
 
 # Set those common args which are at the Perl level.
 sub slv_common_set {
     my ( $slv, $flat_args ) = @_;
     if ( my $value = $flat_args->{'trace_file_handle'} ) {
-        $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE] = $value;
+        $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE] = $value;
     }
     my $trace_file_handle =
-      $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE];
+      $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE];
     delete $flat_args->{'trace_file_handle'};
     return $flat_args;
 }
@@ -61,7 +61,7 @@ sub Marpa::R3::Context::g1_range {
     my ( $start, $end ) =
       $slv->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
 local slv = ...
-return slv.this_step.start_es_id, slv.this_step.es_id
+return slv:g1_range()
 END_OF_LUA
     return $start, $end;
 } ## end sub Marpa::R3::Context::g1_range
@@ -72,8 +72,7 @@ sub Marpa::R3::Context::lc_range {
       $slv->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
 local slv = ...
 local slr = slv.slr
-local g1_first = slv.this_step.start_es_id
-local g1_last = slv.this_step.es_id - 1
+local g1_first, g1_last = slv:g1_range()
 local l0_first_b, l0_first_p = slr:g1_to_block_first(g1_first)
 local l0_last_b, l0_last_p = slr:g1_to_block_last(g1_last)
 return slr:lc_range_brief(l0_first_b, l0_first_p, l0_last_b, l0_last_p)
@@ -86,8 +85,8 @@ sub Marpa::R3::Context::g1_span {
     my ( $start, $length ) =
       $slv->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
 local slv = ...
-local start = slv.this_step.start_es_id + 0
-local length = (start - slv.this_step.es_id) + 1
+local g1_first, g1_last = slv:g1_range()
+local length = g1_last - g1_first + 1
 return start, length
 END_OF_LUA
     return $start, $length;
@@ -195,7 +194,7 @@ sub code_problems {
 
 }
 
-sub Marpa::R3::Scanless::V::new {
+sub Marpa::R3::Valuer::new {
     my ( $class, @args ) = @_;
 
     my $slv = [];
@@ -210,12 +209,12 @@ sub Marpa::R3::Scanless::V::new {
 
     my $slr = $flat_args->{recognizer};
     Marpa::R3::exception(
-        qq{Marpa::R3::Scanless::V::new() called without a "recognizer" argument} )
+        qq{Marpa::R3::Valuer::new() called without a "recognizer" argument} )
       if not defined $slr;
-    $slv->[Marpa::R3::Internal::Scanless::V::SLR] = $slr;
+    $slv->[Marpa::R3::Internal_V::SLR] = $slr;
     delete $flat_args->{recognizer};
 
-    my $slr_class = 'Marpa::R3::Scanless::R';
+    my $slr_class = 'Marpa::R3::Recognizer';
     if ( not blessed $slr or not $slr->isa($slr_class) ) {
         my $ref_type = ref $slr;
         my $desc = $ref_type ? "a ref to $ref_type" : 'not a ref';
@@ -225,14 +224,14 @@ sub Marpa::R3::Scanless::V::new {
         );
     }
 
-    $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE] //=
-      $slr->[Marpa::R3::Internal::Scanless::R::TRACE_FILE_HANDLE];
+    $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE] //=
+      $slr->[Marpa::R3::Internal_R::TRACE_FILE_HANDLE];
 
     my $trace_file_handle =
-      $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE];
+      $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE];
 
-    my $lua = $slr->[Marpa::R3::Internal::Scanless::R::L];
-    $slv->[Marpa::R3::Internal::Scanless::V::L] = $lua;
+    my $lua = $slr->[Marpa::R3::Internal_R::L];
+    $slv->[Marpa::R3::Internal_V::L] = $lua;
 
     my ( $regix ) = $slr->coro_by_tag(
         ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -257,15 +256,15 @@ sub Marpa::R3::Scanless::V::new {
 END_OF_LUA
 
     return if $regix < 0;
-    $slv->[Marpa::R3::Internal::Scanless::V::REGIX]  = $regix;
+    $slv->[Marpa::R3::Internal_V::REGIX]  = $regix;
 
     return bless $slv, $class;
 }
 
-sub Marpa::R3::Scanless::V::DESTROY {
-    # say STDERR "In Marpa::R3::Scanless::V::DESTROY before test";
+sub Marpa::R3::Valuer::DESTROY {
+    # say STDERR "In Marpa::R3::Valuer::DESTROY before test";
     my $slv = shift;
-    my $lua = $slv->[Marpa::R3::Internal::Scanless::V::L];
+    my $lua = $slv->[Marpa::R3::Internal_V::L];
 
     # If we are destroying the Perl interpreter, then all the Marpa
     # objects will be destroyed, including Marpa's Lua interpreter.
@@ -275,9 +274,9 @@ sub Marpa::R3::Scanless::V::DESTROY {
     # In fact, the Lua interpreter may already have been destroyed,
     # so this test is necessary to avoid a warning message.
     return if not $lua;
-    # say STDERR "In Marpa::R3::Scanless::V::DESTROY after test";
+    # say STDERR "In Marpa::R3::Valuer::DESTROY after test";
 
-    my $regix = $slv->[Marpa::R3::Internal::Scanless::V::REGIX];
+    my $regix = $slv->[Marpa::R3::Internal_V::REGIX];
     $slv->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', '');
@@ -287,14 +286,14 @@ sub Marpa::R3::Scanless::V::DESTROY {
 END_OF_LUA
 }
 
-sub Marpa::R3::Scanless::V::set {
+sub Marpa::R3::Valuer::set {
     my ( $slv, @args ) = @_;
 
     my ($flat_args, $error_message) = Marpa::R3::flatten_hash_args(\@args);
     Marpa::R3::exception( sprintf $error_message, '$slv->set()' ) if not $flat_args;
     $flat_args = slv_common_set($slv, $flat_args);
     my $trace_file_handle =
-      $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE];
+      $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE];
 
     $slv->coro_by_tag(
         ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -320,23 +319,23 @@ END_OF_LUA
 }
 
 # Returns false if no parse
-sub Marpa::R3::Scanless::V::value {
+sub Marpa::R3::Valuer::value {
     my ( $slv, $per_parse_arg ) = @_;
-    my $slr    = $slv->[Marpa::R3::Internal::Scanless::V::SLR];
-    my $slg    = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
+    my $slr    = $slv->[Marpa::R3::Internal_V::SLR];
+    my $slg    = $slr->[Marpa::R3::Internal_R::SLG];
 
     my $trace_actions =
-      $slg->[Marpa::R3::Internal::Scanless::G::TRACE_ACTIONS] // 0;
+      $slg->[Marpa::R3::Internal_G::TRACE_ACTIONS] // 0;
     my $trace_file_handle =
-      $slv->[Marpa::R3::Internal::Scanless::V::TRACE_FILE_HANDLE];
+      $slv->[Marpa::R3::Internal_V::TRACE_FILE_HANDLE];
 
-        my $semantics_arg0 = $per_parse_arg // {};
-        my $constants = $slg->[Marpa::R3::Internal::Scanless::G::CONSTANTS];
-        my $null_values = $slg->[Marpa::R3::Internal::Scanless::G::NULL_VALUES];
-        my $nulling_closures =
-          $slg->[Marpa::R3::Internal::Scanless::G::CLOSURE_BY_SYMBOL_ID];
-        my $rule_closures =
-          $slg->[Marpa::R3::Internal::Scanless::G::CLOSURE_BY_RULE_ID];
+    my $semantics_arg0 = $per_parse_arg // {};
+    my $constants = $slg->[Marpa::R3::Internal_G::CONSTANTS];
+    my $null_values = $slg->[Marpa::R3::Internal_G::NULL_VALUES];
+    my $nulling_closures =
+      $slg->[Marpa::R3::Internal_G::CLOSURE_BY_SYMBOL_ID];
+    my $rule_closures =
+      $slg->[Marpa::R3::Internal_G::CLOSURE_BY_RULE_ID];
 
     local $Marpa::R3::Context::rule = undef;
     local $Marpa::R3::Context::irlid = undef;
@@ -367,9 +366,56 @@ sub Marpa::R3::Scanless::V::value {
         },
         bless => sub {
             my ( $value, $blessing_ix ) = @_;
-            my $blessing = $constants->[$blessing_ix];
+            my $blessing_data = $constants->[$blessing_ix];
+            my ( $irlid, $lexeme_id, $blessing ) = @{$blessing_data};
+            my $lexeme_name;
+            if ( defined $lexeme_id ) {
+                $lexeme_name = $slg->g1_symbol_name($lexeme_id);
+            }
+            else {
+                $lexeme_name = "[IRL# $irlid]";
+            }
+
+          FIND_BASE_BLESSING: {
+                if ( $blessing eq '::name' ) {
+                    if ( $lexeme_name =~ / [^ [:alnum:]] /xms ) {
+                        Marpa::R3::exception(
+qq{Lexeme blessing by '::name' only allowed if lexeme name is whitespace and alphanumerics\n},
+                            qq{   Problematic lexeme was <$lexeme_name>\n}
+                        );
+                    } ## end if ( $lexeme_name =~ / [^ [:alnum:]] /xms )
+                    $blessing = $lexeme_name;
+                    $blessing =~ s/[ ]/_/gxms;
+                    last FIND_BASE_BLESSING;
+                } ## end if ( $default_blessing eq '::name' )
+                if ( $blessing =~ /^ :: /xms ) {
+                    Marpa::R3::exception(
+                        qq{Blessing lexeme as '$blessing' is not allowed\n},
+qq{   It is in pseudo-blessing form, but there is no such psuedo-blessing\n},
+                        qq{   Problematic lexeme was <$lexeme_name>\n}
+                    );
+                }
+                if ( $blessing =~ / [\W] /xms ) {
+                    Marpa::R3::exception(
+                        qq{Blessing lexeme as '$blessing' is not allowed\n},
+qq{   It contained non-word characters and that is not allowed\n},
+                        qq{   Problematic lexeme was <$lexeme_name>\n}
+                    );
+                } ## end if ( $default_blessing =~ / [\W] /xms )
+            }
+
+            if ( $blessing !~ / :: /xms ) {
+                my $bless_package =
+                  $slg->[Marpa::R3::Internal_G::BLESS_PACKAGE];
+                if ( not defined $bless_package ) {
+                    Marpa::R3::exception(
+                        qq{Blessing package needed, but grammar has none\n},
+                        qq{  The blessing was "$blessing"\n} );
+                } ## end if ( not defined $bless_package )
+                $blessing = $bless_package . q{::} . $blessing;
+            }
             return 'sig', [ 'S', ( bless $value, $blessing ) ];
-        },
+          },
         perl_nulling_semantics => sub {
             my ($token_id) = @_;
             my $value_ref = $nulling_closures->[$token_id];
@@ -398,7 +444,7 @@ sub Marpa::R3::Scanless::V::value {
                         warnings    => \@warnings,
                         where       => 'computing value',
                         long_where  => 'Computing value for null symbol: '
-                          . $slg->g1_symbol_display($token_id),
+                          . $slg->g1_symbol_display_form($token_id),
                     }
                 );
             } ## end if ( not $eval_ok or @warnings )
@@ -459,10 +505,10 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::call_by_tag {
+sub Marpa::R3::Valuer::call_by_tag {
     my ( $slv, $tag, $codestr, $signature, @args ) = @_;
-    my $lua   = $slv->[Marpa::R3::Internal::Scanless::V::L];
-    my $regix = $slv->[Marpa::R3::Internal::Scanless::V::REGIX];
+    my $lua   = $slv->[Marpa::R3::Internal_V::L];
+    my $regix = $slv->[Marpa::R3::Internal_V::REGIX];
 
     my @results;
     my $eval_error;
@@ -483,10 +529,10 @@ sub Marpa::R3::Scanless::V::call_by_tag {
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::coro_by_tag {
+sub Marpa::R3::Valuer::coro_by_tag {
     my ( $slv, $tag, $args, $codestr ) = @_;
-    my $lua        = $slv->[Marpa::R3::Internal::Scanless::V::L];
-    my $regix      = $slv->[Marpa::R3::Internal::Scanless::V::REGIX];
+    my $lua        = $slv->[Marpa::R3::Internal_V::L];
+    my $regix      = $slv->[Marpa::R3::Internal_V::REGIX];
     my $handler    = $args->{handlers} // {};
     my $resume_tag = $tag . '[R]';
     my $signature  = $args->{signature} // '';
@@ -545,7 +591,7 @@ sub Marpa::R3::Scanless::V::coro_by_tag {
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::tree_show {
+sub Marpa::R3::Valuer::tree_show {
     my ( $slv, $verbose ) = @_;
     my $text = q{};
     NOOK: for ( my $nook_id = 0; 1; $nook_id++ ) {
@@ -557,9 +603,9 @@ sub Marpa::R3::Scanless::V::tree_show {
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::nook_show {
+sub Marpa::R3::Valuer::nook_show {
     my ( $slv, $nook_id, $verbose ) = @_;
-    my $slr = $slv->[Marpa::R3::Internal::Scanless::V::SLR];
+    my $slr = $slv->[Marpa::R3::Internal_V::SLR];
 
     my ($or_node_id, $text) = $slv->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
@@ -636,7 +682,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::and_node_tag {
+sub Marpa::R3::Valuer::and_node_tag {
     my ( $slv, $and_node_id ) = @_;
 
     my ($tag) = $slv->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -649,10 +695,10 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::verbose_or_node {
+sub Marpa::R3::Valuer::verbose_or_node {
     my ( $slv, $or_node_id ) = @_;
-    my $slr = $slv->[Marpa::R3::Internal::Scanless::V::SLR];
-    my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
+    my $slr = $slv->[Marpa::R3::Internal_V::SLR];
+    my $slg = $slr->[Marpa::R3::Internal_R::SLG];
 
     my ($text, $nrl_id, $position)
         = $slv->call_by_tag(
@@ -685,7 +731,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::bocage_show {
+sub Marpa::R3::Valuer::bocage_show {
     my ($slv)     = @_;
 
     my ($result) = $slv->call_by_tag(
@@ -699,7 +745,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::or_nodes_show {
+sub Marpa::R3::Valuer::or_nodes_show {
     my ( $slv ) = @_;
 
     my ($result) = $slv->call_by_tag(
@@ -713,7 +759,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::and_nodes_show {
+sub Marpa::R3::Valuer::and_nodes_show {
     my ( $slv ) = @_;
     my ($result) = $slv->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
@@ -725,21 +771,21 @@ END_OF_LUA
     return $result;
 }
 
-sub Marpa::R3::Scanless::V::ambiguous {
+sub Marpa::R3::Valuer::ambiguous {
     my ($slv) = @_;
-    my $slr = $slv->[Marpa::R3::Internal::Scanless::V::SLR];
+    my $slr = $slv->[Marpa::R3::Internal_V::SLR];
     my $ambiguity_level = $slv->ambiguity_level();
     return q{No parse} if $ambiguity_level <= 0;
     return q{} if $ambiguity_level == 1;
     # ASF must be created for end location of SLV (not SLR!)
-    my $asf = Marpa::R3::ASF->new( { recognizer => $slr, end => $slv->g1_pos() } );
+    my $asf = Marpa::R3::ASF2->new( { recognizer => $slr, end => $slv->g1_pos() } );
     die 'Could not create ASF' if not defined $asf;
-    my $ambiguities = Marpa::R3::Internal::ASF::ambiguities($asf);
+    my $ambiguities = Marpa::R3::Internal_ASF2::ambiguities($asf);
     my @ambiguities = grep {defined} @{$ambiguities}[ 0 .. 1 ];
-    return Marpa::R3::Internal::ASF::ambiguities_show( $asf, \@ambiguities );
-} ## end sub Marpa::R3::Scanless::R::ambiguous
+    return Marpa::R3::Internal_ASF2::ambiguities_show( $asf, \@ambiguities );
+} ## end sub Marpa::R3::Recognizer::ambiguous
 
-sub Marpa::R3::Scanless::V::ambiguity_level {
+sub Marpa::R3::Valuer::ambiguity_level {
     my ($slv) = @_;
 
     my ($metric) = $slv->call_by_tag(
@@ -751,7 +797,7 @@ END__OF_LUA
     return $metric;
 }
 
-sub Marpa::R3::Scanless::V::g1_pos {
+sub Marpa::R3::Valuer::g1_pos {
     my ( $slv ) = @_;
     my ($g1_pos) = $slv->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
@@ -763,9 +809,9 @@ END__OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::V::regix {
+sub Marpa::R3::Valuer::regix {
     my ( $slv ) = @_;
-    my $regix = $slv->[Marpa::R3::Internal::Scanless::V::REGIX];
+    my $regix = $slv->[Marpa::R3::Internal_V::REGIX];
     return $regix;
 }
 
